// Demo3D.java

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

public class Demo3D extends JApplet
{
   private static final int MINX = -50;
   private static final int MAXX = 50;
   private static final int DEFX = 6;

   private static final int MINY = -50;
   private static final int MAXY = 50;
   private static final int DEFY = 8;

   private static final int MINZ = -50;
   private static final int MAXZ = 50;
   private static final int DEFZ = 7;

   // Viewpoint position variables

   private double posX = DEFX, posY = DEFY, posZ = DEFZ;

   // Perspective variable -- ds represents the D/S perspective ratio in my
   // computer graphics textbook. This variable defaults to halfway between
   // 0 degrees (extreme telephoto) and 180 degrees (extreme wide-angle).

   private double ds = 1.0 / Math.tan (Math.toRadians (45.0));

   public void init ()
   {
      getContentPane ().add (buildGUI ());

      // After installing the GUI, create an empty border around the GUI and
      // make sure that its color matches the slider control's background
      // color -- to keep the GUI looking nice under different look and feels.

      getRootPane ().setBorder (BorderFactory.createEmptyBorder (10, 10, 0, 10));
      getRootPane ().setOpaque (true);
      getRootPane ().setBackground ((Color) UIManager.get ("Slider.background"));
   }

   private JPanel buildGUI ()
   {
      // Although I'm ignoring the computer screen's aspect ratio (I don't care
      // if individual pixels aren't square), I want the applet's dimensions to 
      // be square.

      int width = getWidth ();
      if (width != getHeight ())
          return null;

      // Create a 3D panel where each side is 60% of the applet's width. Set
      // the default viewpoint from which an observer views the scene. Also,
      // set the default perspective to halfway between extreme telephoto and 
      // extreme wide-angle.

      final Panel3D p3d = new Panel3D (width*3/5);
      p3d.lookFrom (posX, posY, posZ);
      p3d.perspective (ds);

      // GUI components are stored in the following panel.

      JPanel panel = new JPanel ();
      panel.setLayout (new BorderLayout ());

      // The top panel contains the X and Y slider controls; it also contains
      // the 3D panel.

      JPanel panelTop = new JPanel ();

      // Build the top panel.

      JPanel panelTemp = new JPanel ();
      panelTemp.add (new JLabel ("X"));

      final JSlider sliderX = new JSlider (JSlider.VERTICAL, MINX, MAXX, DEFX);
      sliderX.setMinorTickSpacing (5);
      sliderX.setMajorTickSpacing (10);
      sliderX.setPaintTicks (true);
      sliderX.setPaintLabels (true);
      sliderX.setLabelTable (sliderX.createStandardLabels (10));
      ChangeListener cl;
      cl = new ChangeListener ()
           {
               public void stateChanged (ChangeEvent e)
               {
                  posX = sliderX.getValue ();
                  p3d.lookFrom (posX, posY, posZ);
                  p3d.perspective (ds);
               }
           };
      sliderX.addChangeListener (cl);
      panelTemp.add (sliderX);

      panelTop.add (panelTemp);

      panelTemp = new JPanel ();
      panelTemp.add (p3d);

      panelTop.add (panelTemp);

      panelTemp = new JPanel ();
      panelTemp.add (new JLabel ("Y"));

      final JSlider sliderY = new JSlider (JSlider.VERTICAL, MINY, MAXY, DEFY);
      sliderY.setMinorTickSpacing (5);
      sliderY.setMajorTickSpacing (10);
      sliderY.setPaintTicks (true);
      sliderY.setPaintLabels (true);
      sliderY.setLabelTable (sliderY.createStandardLabels (10));
      cl = new ChangeListener ()
           {
               public void stateChanged (ChangeEvent e)
               {
                  posY = sliderY.getValue ();
                  p3d.lookFrom (posX, posY, posZ);
                  p3d.perspective (ds);
               }
           };
      sliderY.addChangeListener (cl);
      panelTemp.add (sliderY);

      panelTop.add (panelTemp);

      // Place the top panel in the top area of the overall panel.

      panel.add (panelTop, BorderLayout.NORTH);

      // The bottom panel contains the Z slider control; it also contains the
      // field of view slider and a panel of buttons.

      JPanel panelBottom = new JPanel ();
      panelBottom.setLayout (new BorderLayout ());

      // Build the bottom panel.

      panelTemp = new JPanel ();
      panelTemp.add (new JLabel ("Z"));

      final JSlider sliderZ = new JSlider (JSlider.HORIZONTAL, MINZ, MAXZ, DEFZ);
      sliderZ.setMinorTickSpacing (5);
      sliderZ.setMajorTickSpacing (10);
      sliderZ.setPaintTicks (true);
      sliderZ.setPaintLabels (true);
      sliderZ.setLabelTable (sliderZ.createStandardLabels (10));
      cl = new ChangeListener ()
           {
               public void stateChanged (ChangeEvent e)
               {
                  posZ = sliderZ.getValue ();
                  p3d.lookFrom (posX, posY, posZ);
                  p3d.perspective (ds);
               }
           };
      sliderZ.addChangeListener (cl);
      panelTemp.add (sliderZ);

      panelBottom.add (panelTemp, BorderLayout.NORTH);

      JPanel panelFOV = new JPanel ();
      panelFOV.setLayout (new BorderLayout ());

      panelFOV.add (new JLabel ("Field Of View", JLabel.CENTER),
                    BorderLayout.CENTER);

      final JSlider sliderFOV = new JSlider (JSlider.HORIZONTAL, 0, 180, 90);
      sliderFOV.setMinorTickSpacing (5);
      sliderFOV.setMajorTickSpacing (10);
      sliderFOV.setPaintTicks (true);
      sliderFOV.setPaintLabels (true);
      sliderFOV.setLabelTable (sliderFOV.createStandardLabels (10));
      cl = new ChangeListener ()
           {
               public void stateChanged (ChangeEvent e)
               {
                  int angle = sliderFOV.getValue ();

                  if (angle == 0)
                      ds = Double.MAX_VALUE;
                  else
                  if (angle == 180)
                      ds = 0.0;
                  else
                  {
                      double rads;
                      rads = Math.toRadians (angle / 2.0);
                      ds = 1.0 / Math.tan (rads);
                  }

                  p3d.lookFrom (posX, posY, posZ);
                  p3d.perspective (ds);
               }
           };
      sliderFOV.addChangeListener (cl);
      panelFOV.add (sliderFOV, BorderLayout.SOUTH);

      panelBottom.add (panelFOV, BorderLayout.CENTER);

      JButton btnCube = new JButton ("Cube");
      ActionListener al;
      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      p3d.load ("cube.3d");
                  }
                  catch (Exception e2)
                  {
                      JOptionPane.showMessageDialog (Demo3D.this,
                                                     e2.getMessage ());
                  }
               }
           };
      btnCube.addActionListener (al);

      JButton btnPyramid = new JButton ("Pyramid");
      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      p3d.load ("pyramid.3d");
                  }
                  catch (Exception e2)
                  {
                      JOptionPane.showMessageDialog (Demo3D.this,
                                                     e2.getMessage ());
                  }
               }
           };
      btnPyramid.addActionListener (al);

      btnCube.setPreferredSize (btnPyramid.getPreferredSize ());

      JButton btnTower = new JButton ("Tower");
      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      p3d.load ("tower.3d");
                  }
                  catch (Exception e2)
                  {
                      JOptionPane.showMessageDialog (Demo3D.this,
                                                     e2.getMessage ());
                  }
               }
           };
      btnTower.addActionListener (al);

      btnTower.setPreferredSize (btnPyramid.getPreferredSize ());

      panelTemp = new JPanel ();
      panelTemp.setLayout (new FlowLayout (FlowLayout.CENTER, 15, 15));

      panelTemp.add (btnCube);
      panelTemp.add (btnPyramid);
      panelTemp.add (btnTower);

      panelBottom.add (panelTemp, BorderLayout.SOUTH);

      // Place the bottom panel in the center area of the overall panel to
      // occupy empty space below the top area of the overall panel.

      panel.add (panelBottom, BorderLayout.CENTER);

      return panel;
   }
}
